"use strict";

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var AdharaApp = function () {
    function AdharaApp() {
        _classCallCheck(this, AdharaApp);
    }

    _createClass(AdharaApp, [{
        key: "getEntityConfig",


        /**
         * @function
         * @instance
         * @return {Object} Adhara style entity config
         * */
        value: function getEntityConfig(context_name) {
            var context = this.config[context_name];
            var allowed_query_types = context.data_config.allowed_query_types ? context.data_config.allowed_query_types.slice() : [];
            var data_config = void 0;
            if (context.data_config.hasOwnProperty("batch_data_override")) {
                data_config = {
                    batch_data_override: context.data_config.batch_data_override.map(function (batch_data_config) {
                        return {
                            url: batch_data_config.url,
                            query_type: batch_data_config.query_type,
                            identifier: batch_data_config.identifier,
                            reuse: batch_data_config.reuse,
                            blob: batch_data_config.blob
                        };
                    })
                };
            } else {
                data_config = {
                    url: context.data_config.url,
                    allowed_query_types: allowed_query_types,
                    default_query_type: context.data_config.default_query_type || allowed_query_types[0],
                    socket_tag: context.data_config.socket_tag,
                    reuse: context.data_config.reuse,
                    blob: context.data_config.blob
                };
            }
            return {
                data_config: data_config,
                view: context.view,
                processor: context.processor
            };
        }

        /**
         * @getter
         * @instance
         * @returns {Object} Custom global view configurations
         * */

    }, {
        key: "toast",


        /**
         * @function
         * @instance
         * @param {String} title - toast message title
         * @param {String} content - toast message content
         * @param {String} type - toast message type. can take the values "success"|"error"|"info".
         * */
        value: function toast(title, content, type) {
            AdharaDefaultToaster.make(title, content, type);
        }
    }, {
        key: "debug",


        /**
         * @function
         * @instance
         * @return {Boolean} Whether the app is being accessed in development mode or in production mode
         * */
        get: function get() {
            return ["localhost", "127.0.0.1", "0.0.0.0"].indexOf(window.location.hostname) !== -1 || window.location.hostname.indexOf("192.168.") !== -1;
        }

        /**
         * @function
         * @instance
         * @return {String} App name
         * */

    }, {
        key: "name",
        get: function get() {
            return "Adhara App";
        }

        /**
         * @function
         * @instance
         * @return {Object} App name in detail. Like first and last name
         * */

    }, {
        key: "detailedName",
        get: function get() {
            return {
                first: "Adhara",
                last: "App"
            };
        }

        /**
         * @function
         * @instance
         * @return {String} Tag line
         * */

    }, {
        key: "tagLine",
        get: function get() {}

        /**
         * @function
         * @instance
         * @return {AdharaView} Container view class
         * */

    }, {
        key: "containerView",
        get: function get() {}

        /**
         * @function
         * @instance
         * @return {Object} Adhara style app config
         * */

    }, {
        key: "config",
        get: function get() {
            return {};
        }
    }, {
        key: "customViewConfig",
        get: function get() {
            return {
                fetching_data: {
                    AdharaListView: "Fetching Data..."
                },
                no_data: {
                    AdharaListView: "No Data Available"
                }
            };
        }

        /**
         * @function
         * @instance
         * @returns {Array<String>} list of http methods to be allowed by the application.
         * @description This getter can be configured to return allowed methods based on the current network state.
         * Say if offline, it can be configured to just return `["get"]` method which will restrict DataInterface from making
         * other service API calls such as "post", "delete", etc...
         * */

    }, {
        key: "allowedHttpMethods",
        get: function get() {
            return ['get', 'post', 'put', 'delete']; // all available API methods
            // offline mode will switch a few of these off (post, put and delete)
        }

        /**
         * @function
         * @instance
         * @return {AdharaRouterConfiguration} Adhara style routing config
         * */

    }, {
        key: "routerConfiguration",
        get: function get() {
            return {
                routes: [],
                on_route_listeners: {},
                middlewares: []
            };
        }

        /**
         * @function
         * @instance
         * @returns {String} A css selector in which app is to be rendered.
         * */

    }, {
        key: "DOMSelector",
        get: function get() {
            return "app"; //=> search DOM for `<app></app>`
        }

        /**
         * @function
         * @instance
         * @returns {String} API Server URL. Either the base path or a full url till base path.
         * */

    }, {
        key: "apiServerURL",
        get: function get() {
            return "/api";
        }

        /**
         * @function
         * @instance
         * @returns {Object} WebSocket config object.
         * sample...
         * {
         *  url: "sub.domain.com:9081"
         * }
         * */

    }, {
        key: "webSocketConfig",
        get: function get() {}
    }]);

    return AdharaApp;
}();
//# sourceMappingURL=App.js.map