"use strict";

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * @class
 * @classdesc a base class that is to be extended by all the view classes
 * */
var AdharaView = function (_AdharaController) {
    _inherits(AdharaView, _AdharaController);

    /**
     * @constructor
     * @param {AdharaView} parentViewInstance - parent view instance that is to be passed to render a view inside another.
     * */
    function AdharaView(parentViewInstance) {
        _classCallCheck(this, AdharaView);

        var _this = _possibleConstructorReturn(this, (AdharaView.__proto__ || Object.getPrototypeOf(AdharaView)).call(this));

        _this._parentView = parentViewInstance;
        Adhara.instances[_this.constructor.name] = _this;
        _this._data = null;
        _this._state = {};
        _this._error = null;
        _this._event_listeners = {};
        _this._registerEvents(["ViewRendered", "SubViewsRendered", "ViewFormatted"]);
        _this._registerEvents(_this.events);
        _this.onInit();
        return _this;
    }

    /**
     * @function
     * @instance
     * @description Anything that needs to be done right after initializing the view.
     * */


    _createClass(AdharaView, [{
        key: "onInit",
        value: function onInit() {}
        //Override this method to do any miscellaneous operations/assignments right after initializing the View


        /**
         * @getter
         * @instance
         * @returns {Array<String>} return list of custom event names in PascalCase...
         * @description event listener registration functions will be created dynamically by prepending "on" th the event name
         * @example
         * say get events() returns this array: ["Render", "Format"]
         * listener functions these events can be then registered using `onRender(listener)` and `onFormat(listener)`
         * */

    }, {
        key: "_registerEvents",


        /**
         * @function
         * @protected
         * @param {Array<String>} event_names - list of events that are to be enabled on the current View instance.
         * @description registers the event names and creates event registration functions.
         * @example
         * _registerEvents(["Render", "Format"]);
         * //This will create 2 functions onRender and onFormat runtime to allow registration of events
         * */
        value: function _registerEvents(event_names) {
            var _this2 = this;

            var _iteratorNormalCompletion = true;
            var _didIteratorError = false;
            var _iteratorError = undefined;

            try {
                var _loop = function _loop() {
                    var event_name = _step.value;

                    _this2._event_listeners[event_name] = [];
                    _this2["on" + event_name] = function (handler) {
                        _this2._event_listeners[event_name].push(handler);
                    };
                    _this2["off" + event_name] = function (handler) {
                        _this2._event_listeners[event_name].splice(_this2._event_listeners[event_name].indexOf(handler), 1);
                    };
                };

                for (var _iterator = event_names[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                    _loop();
                }
            } catch (err) {
                _didIteratorError = true;
                _iteratorError = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion && _iterator.return) {
                        _iterator.return();
                    }
                } finally {
                    if (_didIteratorError) {
                        throw _iteratorError;
                    }
                }
            }
        }
    }, {
        key: "trigger",
        value: function trigger(event_name) {
            for (var _len = arguments.length, data = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
                data[_key - 1] = arguments[_key];
            }

            var _iteratorNormalCompletion2 = true;
            var _didIteratorError2 = false;
            var _iteratorError2 = undefined;

            try {
                for (var _iterator2 = this._event_listeners[event_name][Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
                    var event_handler = _step2.value;

                    event_handler(data);
                }
            } catch (err) {
                _didIteratorError2 = true;
                _iteratorError2 = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion2 && _iterator2.return) {
                        _iterator2.return();
                    }
                } finally {
                    if (_didIteratorError2) {
                        throw _iteratorError2;
                    }
                }
            }
        }

        /**
         * @method
         * @getter
         * @returns {HandlebarTemplate} template of the view
         * */

    }, {
        key: "getCustomTemplate",
        value: function getCustomTemplate(type) {
            var _iteratorNormalCompletion3 = true;
            var _didIteratorError3 = false;
            var _iteratorError3 = undefined;

            try {
                for (var _iterator3 = Adhara.viewHierarchy[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
                    var ViewClass = _step3.value;

                    if (ViewClass.isPrototypeOf(this.constructor)) {
                        if (Adhara.app.customViewConfig[type][ViewClass.name]) {
                            return Adhara.app.customViewConfig[type][ViewClass.name];
                        }
                    }
                }
            } catch (err) {
                _didIteratorError3 = true;
                _iteratorError3 = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion3 && _iterator3.return) {
                        _iterator3.return();
                    }
                } finally {
                    if (_didIteratorError3) {
                        throw _iteratorError3;
                    }
                }
            }
        }

        /**
         * @method
         * @getter
         * @returns {HandlebarTemplate} structure to be rendered when data is being fetched
         * */

    }, {
        key: "create",
        value: function create() {
            Adhara.addToActiveViews(this);
            this.fetchData();
        }
    }, {
        key: "isActive",
        value: function isActive() {
            return Adhara.isActiveView(this);
        }

        /**
         * @function
         * @instance
         * @description hook to make API calls, data change event will be triggered on successful API call.
         * By default no API call will be made and dataChange method will be called right away.
         * */

    }, {
        key: "fetchData",
        value: function fetchData() {
            this._state.fetching_data = true;
            var config = this.entityConfig;
            if (config) {
                if (config.data_config.hasOwnProperty("batch_data_override")) {
                    this.render();
                    return this.control(undefined, config);
                }
                if (dataInterface.getHTTPMethod(config.data_config.default_query_type) === "get") {
                    this.render();
                    return this.control(config.data_config.default_query_type, config, this.payload);
                }
                return this.handleDataChange();
            }
            this.handleDataChange();
        }

        /**
         * @function
         * @private
         * @param {DataBlob|Array<DataBlob>} [new_data=null] - Data in the form of DataBlob instance to be updated in view
         * */

    }, {
        key: "handleDataChange",
        value: function handleDataChange(new_data) {
            if (!this.isActive()) {
                return;
            }
            this.dataChange(new_data);
            this._state.fetching_data = false;
            this.render();
        }

        /**
         * @function
         * @private
         * @param {*} error - Error to be updated in view
         * */

    }, {
        key: "handleDataError",
        value: function handleDataError(error) {
            if (!this.isActive()) {
                return;
            }
            this.dataError(error);
            this._state.fetching_data = false;
            this.render();
        }

        /**
         * @function
         * @private
         * @param {Object<String, Object<String, DataBlob|*>>} map - Data/Error to be updated in view. Map of {String} identifier vs {Object<success:response|error:error>}
         * */

    }, {
        key: "handleBatchData",
        value: function handleBatchData(map) {
            if (!this.isActive()) {
                return;
            }
            var errors = {};
            var success = {};
            for (var identifier in map) {
                if (map.hasOwnProperty(identifier)) {
                    if (map[identifier].hasOwnProperty("error")) {
                        errors[identifier] = map[identifier].error;
                    } else {
                        success[identifier] = map[identifier].success;
                    }
                }
            }
            this.dataChange(success);
            if (Object.keys(errors).length) {
                this.dataError(errors);
            }
            this._state.fetching_data = false;
            this.render();
        }

        /**
         * @function
         * @instance
         * @param {DataBlob|*} new_data - Data in the form of DataBlob instance to be set in view
         * */

    }, {
        key: "dataChange",
        value: function dataChange(new_data) {
            this._data = new_data;
            this._error = null;
        }

        /**
         * @function
         * @instance
         * @param {*} error - Error to be set in view
         * */

    }, {
        key: "dataError",
        value: function dataError(error) {
            this._error = error;
        }

        /**
         * @method
         * @getter
         * @returns {*} View errors, that can be consumed by the template.
         * */

    }, {
        key: "_getHTML",
        value: function _getHTML(template) {
            return HandlebarUtils.execute(template || (this.state.fetching_data ? this.fetchingDataTemplate : this.template), this);
        }
    }, {
        key: "_getParentContainer",
        value: function _getParentContainer() {
            var container = this.parentView.contentContainer;
            if (typeof container === "string") {
                return container;
            }
            return container[this.constructor.name] || container["*"];
        }
    }, {
        key: "getParentContainerElement",
        value: function getParentContainerElement() {
            return document.querySelector(this._getParentContainer());
        }
    }, {
        key: "_render",
        value: function _render() {}
    }, {
        key: "render",
        value: function render() {
            var _this3 = this;

            var container = this.getParentContainerElement();
            if (!container) {
                console.warn("No container defined/available", this.constructor.name);
                return;
            }
            container.innerHTML = this._getHTML();
            if (this.state.fetching_data) {
                return;
            }
            this.trigger("ViewRendered");
            this._format(container);
            setTimeout(function () {
                _this3.format(container);
                _this3.trigger("ViewFormatted");
            }, 0);
            this.renderSubViews();
            this.trigger("SubViewsRendered");
        }

        /**
         * @function
         * @getter
         * @returns {String} A CSS selector inside which child views are to be rendered.
         * */

    }, {
        key: "renderSubViews",
        value: function renderSubViews() {
            var _iteratorNormalCompletion4 = true;
            var _didIteratorError4 = false;
            var _iteratorError4 = undefined;

            try {
                for (var _iterator4 = this.subViews[Symbol.iterator](), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
                    var sub_view = _step4.value;

                    Adhara.createView(Adhara.getView(sub_view, this));
                }
            } catch (err) {
                _didIteratorError4 = true;
                _iteratorError4 = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion4 && _iterator4.return) {
                        _iterator4.return();
                    }
                } finally {
                    if (_didIteratorError4) {
                        throw _iteratorError4;
                    }
                }
            }
        }
    }, {
        key: "_format",
        value: function _format(container) {
            var _this4 = this;

            var onClickElements = container.querySelectorAll("[data-onclick]");
            var _iteratorNormalCompletion5 = true;
            var _didIteratorError5 = false;
            var _iteratorError5 = undefined;

            try {
                var _loop2 = function _loop2() {
                    var onClickElement = _step5.value;

                    onClickElement.addEventListener("click", function (event) {
                        var data = onClickElement.dataset;
                        if (_this4[data.onclick]) {
                            _this4[data.onclick](event, data, _this4);
                        } else {
                            getValueFromJSON(window, data.onclick)(event, data, _this4);
                        }
                    });
                };

                for (var _iterator5 = onClickElements[Symbol.iterator](), _step5; !(_iteratorNormalCompletion5 = (_step5 = _iterator5.next()).done); _iteratorNormalCompletion5 = true) {
                    _loop2();
                }
            } catch (err) {
                _didIteratorError5 = true;
                _iteratorError5 = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion5 && _iterator5.return) {
                        _iterator5.return();
                    }
                } finally {
                    if (_didIteratorError5) {
                        throw _iteratorError5;
                    }
                }
            }
        }
    }, {
        key: "format",
        value: function format(container) {
            //Control the DOM elements after rendering
        }
    }, {
        key: "refresh",
        value: function refresh() {
            Adhara.createView(this);
        }
    }, {
        key: "events",
        get: function get() {
            return [];
        }
    }, {
        key: "template",
        get: function get() {
            if (AdharaRouter.getCurrentPageName()) {
                return AdharaRouter.getCurrentPageName().replace(/_/g, "-");
            } else {
                var className = this.constructor.name.toLowerCase();
                if (className.endsWith("view")) {
                    return className.slice(0, className.length - 4);
                } else {
                    return className;
                }
            }
        }
    }, {
        key: "fetchingDataTemplate",
        get: function get() {
            return this.getCustomTemplate("fetching_data") || "";
        }

        /**
         * @method
         * @getter
         * @returns {HandlebarTemplate} structure to be rendered when there is no data available
         * */

    }, {
        key: "noDataTemplate",
        get: function get() {
            return this.getCustomTemplate("no_data") || "";
        }

        /**
         * @function
         * @instance
         * @returns Adhara style entity config for entity mapped with this view
         * */

    }, {
        key: "entityConfig",
        get: function get() {
            var entity_name = Adhara.view_context[this.constructor.name];
            return entity_name ? Adhara.app.getEntityConfig(entity_name) : null;
        }

        /**
         * @function
         * @instance
         * @returns Add dynamic input data to API calls made to server
         * */

    }, {
        key: "payload",
        get: function get() {
            return null;
        }

        /**
         * @getter
         * @instance
         * @returns {Object} state object of the current View
         * */

    }, {
        key: "state",
        get: function get() {
            return this._state;
        }
    }, {
        key: "errors",
        get: function get() {
            return this._error;
        }

        /**
         * @method
         * @getter
         * @returns {*} View data, that can be consumed by the template.
         * */

    }, {
        key: "data",
        get: function get() {
            return this._data;
        }
    }, {
        key: "parentView",
        get: function get() {
            return this._parentView || Adhara.getView(Adhara.app.containerView);
        }
    }, {
        key: "contentContainer",
        get: function get() {
            throw new Error("implement this method");
        }

        /**
         * @typedef {Object} SubView - Sub view configuration
         * @property {String} container_selector - container's CSS selector
         * @property {class} view - view class reference of the sub view
         * */

        /**
         * @function
         * @instance
         * @returns Array<SubView>
         * */

    }, {
        key: "subViews",
        get: function get() {
            return [];
        }
    }]);

    return AdharaView;
}(AdharaController);
//# sourceMappingURL=View.js.map